#!/usr/bin/env python

# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license. 
# Thank you for using free software!

#  SlideshowScreenlet (c) Whise <helder.fraga@hotmail.com>


import screenlets
from screenlets.options import FileOption, IntOption, FloatOption, StringOption, BoolOption, ColorOption
from screenlets import DefaultMenuItem , utils
import cairo
import gtk
import pango
import gobject
import os
import threading
import commands
import random
from urllib2 import urlopen
#from urllib import FancyURLopener
from screenlets import Plugins



Flickr = Plugins.importAPI('Flickr')

log = screenlets.logger.get_default_logger()

class SlideshowScreenlet (screenlets.Screenlet):
	"""A dynamic picture frame that displays either pictures from Flickr or any Media RSS picture feed (e.g. Picasa, deviantART, ...) or a folder in your filesystem. You can switch the picture in the frame anytime by dragging and dropping an image."""
	
	# --------------------------------------------------------------------------
	# meta-info, options
	# --------------------------------------------------------------------------
	
	__name__		= 'SlideshowScreenlet'
	__version__		= '1.2'
	__author__		= 'Jesper Henriksen, Peter Ward aka flowblok <peteraward@gmail.com>, Helder Fraga aka Whise, Deviant part by Mamie, RSS by boamaod'
	__requires__		= ['python-imaging']
	__desc__		= __doc__

	__notifier = None
	
	# attributes
	__image = None
	__timeout = None

	__lock = threading.Lock()

	# editable options
	update_interval = 60

	image_scale		= 1.0

	image_offset_x	= 0
	image_offset_y	= 0

	image_border = 4
	border_color = (0,0,0,0.5)

	image_shadow = 4
	shadow_color = (0,0,0,0.7)

	bounding_x = 200
	bounding_y = 200
	show_bounding = False

	anchor = 'upper left'

	border_r = 0.0
	border_g = 0.0
	border_b = 0.0
	border_a = 0.8

	menu_x = 0
	menu_y = 0

	url = ''
	initialized = False
	updated = False
	slide = True
	echo_args = commands.getoutput('grep "^XDG_PICTURES_DIR" ~/.config/user-dirs.dirs | sed -r -e "s/([^=]*=)(.*)/\\2/"')
	if len(echo_args) <= 0:
		folders = commands.getoutput("echo $HOME")
	else:
		folders = commands.getoutput("echo " + echo_args)
	use_types = ['.jpg', '.gif', '.png','.bmp', '.svg', '.jpeg', '.tif', '.tiff']
	engine = 'Media RSS'
	engine1 = 'Media RSS'
	engine_sel = ['Folder', 'Flickr', 'Media RSS']
	frame_anchor = ['upper left', 'upper right', 'lower left', 'lower right', 'center']
	paint_menu = False
	showbuttons = True
	flickr = None
	flickr_gen = None
	flickr_imgs = None
	flickr_current = ''
	factor = 1
	#preserve_aspect = 0
	recursive = False
	recursion_limit = 400

	flickrurl = 'http://www.flickr.com/explore/interesting/7days/'
	mediaRSS_URL = 'http://backend.deviantart.com/rss.xml?q=boost%3Apopular+in%3Aphotography+max_age%3A24h&type=deviation&offset=0'

	failstate = False
	laststate = False

	# --------------------------------------------------------------------------
	# constructor and internals
	# --------------------------------------------------------------------------

	fetching = False

	def __init__ (self, **keyword_args):
		# call super (and enable drag/drop)
		screenlets.Screenlet.__init__(self, width=200, height=200,
			uses_theme=True, drag_drop=True, **keyword_args)
		# set theme
		self.theme_name = "default"
		# initially apply default image (for newly created instances)
		#self.__image = screenlets.PATH + '/Picframe/dali.png'
		# add default menuitems (all the standard ones)
		self.add_default_menuitems(DefaultMenuItem.XML)

		self.__notifier = screenlets.utils.Notifier(self)
		
		# add option group to properties-dialog
		self.add_options_group('SlideShow', 'Slideshow-related settings ...')
		# add editable options
		#self.add_option(FileOption('Slideshow', '__image', 
		#	self.__image, 'Filename',

		self.add_option(IntOption('SlideShow', 'update_interval', 
			self.update_interval, 'Update interval', 
			'The interval for updating info (in seconds: 3660 = 1 day, 25620 = 1 week)', min=1, max=25620))

		self.add_option(StringOption('SlideShow', 'engine', self.engine,'Select Engine', '',choices = self.engine_sel),realtime=False)

		self.add_option(StringOption('SlideShow', 'flickrurl', self.flickrurl,'Flickr URL', 'Flickr URL'))
		self.add_option(StringOption('SlideShow', 'mediaRSS_URL', self.mediaRSS_URL,'Media RSS URL', 'RSS that publishes images in Media RSS format, Picasa album, deviantART category etc'))
		self.add_option(StringOption('SlideShow', 'folders', self.folders,'Image folder', 'The folder where the pictures are to be found',))
		self.add_option(BoolOption('SlideShow', 'recursive',bool(self.recursive), 'Folder recursion','Show images from subfolders. Use with care, because there could be millions of images in the home folder and it may freeze the application to go through all of them.'))
		self.add_option(IntOption('SlideShow', 'recursion_limit', self.recursion_limit, 'Recursion limit','Max images to include in slideshow material when recurring through the folders.', min=0, max=99999999))
		self.add_option(BoolOption('SlideShow', 'showbuttons',bool(self.showbuttons), 'Show controls','Show contol buttons on focus'))

		self.add_option(IntOption('SlideShow', 'border', 
			self.image_border, 'Frame size', 
			'Size of frame', min=0, max=40))

		self.add_option(ColorOption('SlideShow','border_color', 
			self.border_color, 'Frame Color', ''))

		self.add_option(IntOption('SlideShow', 'shadow', 
			self.image_shadow, 'Shadow size', 
			'Size of shadow', min=1, max=40))

		self.add_option(ColorOption('SlideShow','shadow_color', 
			self.shadow_color, 'Shadow Color', ''))

		self.add_option(IntOption('SlideShow', 'bounding-x', 
			self.bounding_x, 'Bounding X', 
			'X size of bounding box', min=50, max=1600))

		self.add_option(IntOption('SlideShow', 'bounding-y', 
			self.bounding_y, 'Bounding Y', 
			'Y size of bounding box', min=50, max=1200))

		self.add_option(BoolOption('SlideShow', 'showbounding',
		  bool(self.showbuttons), 'Show Boundingbox','Show the bounding box'))

		self.add_option(StringOption('SlideShow', 
		  'anchor', 
		  self.anchor,
		  'Select anchor corner', 
		  'Select anchor corner',
		  choices = self.frame_anchor))

		self.add_option(FloatOption('SlideShow', 'image_scale', self.image_scale, 
			'Image Scale', 'Scale of image within this Picframe ...', 
			min=0.01, max=10.0, digits=2, increment=0.01,hidden=True))

		self.add_option(IntOption('SlideShow', 'image_offset_x', 
			self.image_offset_x, 'Image Offset X', 'X-offset of upper left '+\
			'corner of the image within this Picframe ...', 
			min=0, max=self.width,hidden=True))

		self.add_option(IntOption('SlideShow', 'image_offset_y', 
			self.image_offset_y, 'Image Offset Y', 'Y-offset of upper left '+\
			'corner of the image within this Picframe ...', 
			min=0, max=self.height,hidden=True))

		self.update_interval = self.update_interval
		self.engine = self.engine
		self.folders = self.folders

	def __setattr__ (self, name, value):
		screenlets.Screenlet.__setattr__(self, name, value)

		if name == 'anchor':
		  self.redraw_canvas()

		if name == 'border_color':
			self.redraw_canvas()

		if name == 'border':
			self.image_border = value
			self.redraw_canvas()

		if name == 'shadow_color':
			self.redraw_canvas()

		if name == 'shadow':
			self.image_shadow = value
			self.redraw_canvas()

		if name == 'showbounding':
			self.show_bounding = value
			self.redraw_canvas()

		if name == 'bounding-x':
			self.bounding_x = value
			self.width = self.bounding_x
			self.redraw_canvas()

		if name == 'bounding-y':
			self.bounding_y = value
			self.height = self.bounding_y
			self.redraw_canvas()

		if name == 'showbuttons':
			self.redraw_canvas()
		if name == 'engine':
			if value == 'Folder' :
				self.engine1 = 'Folder'
				self.update()
			if value == '' :
				self.engine1 = 'Folder'
				self.update()
			if value == 'Flickr':
				self.engine1 = value
				self.update()
			if value == 'Media RSS':
				self.engine1 = value
				self.update()
		if name == 'folders' and self.engine == 'Folder':
				self.engine1 = 'Folder'
				self.update()
		if name == "__image":
			screenlets.Screenlet.__setattr__(self, name, value)
			# update view
			self.redraw_canvas()
			#self.update_shape()
		
		if name == "update_interval":
			if value > 0:
				self.__dict__['update_interval'] = value
				if self.__timeout:
					gobject.source_remove(self.__timeout)
				self.__timeout = gobject.timeout_add(int(value * 1000), self.update)
			else:
				self.__dict__['update_interval'] = 1
				pass

	def on_init(self):
		self.update()
		log.debug("Screenlet has been initialized.")
		# add default menuitems
		self.add_default_menuitems()
		self.initialized = True

		self.redraw_canvas()

	def fetch_flickr(self):

		print "Trying Flickr..."

		try:

			if not self.flickr:
				self.flickr = Flickr.Flickr()

			use_new_fetching = hasattr(self.flickr, 'fetch_images')

			if self.flickr_current != self.flickrurl:
				self.flickr_imgs = []
				self.flickr_current = self.flickrurl
				self.flickr_gen = None

			log.debug('Loading %s.' % self.flickrurl)
			# Keep the number of cached images above 20 if possible.
			# This ensures that the Slideshow doesn't get too repetitive.
			while not self.flickr_imgs or len(self.flickr_imgs) < 20:
				if use_new_fetching:
					log.debug('Using new fetching mechanism.')
					# Try to use the previous generator.
					if self.flickr_gen:
						try:
							imgs = self.flickr_gen.next()
						except StopIteration:
							self.flickr_gen = None
		
					# Try a new generator (in case the page updated).
					if not self.flickr_gen:
						self.flickr_gen = self.flickr.fetch_images(self.flickrurl)
						try:
							imgs = self.flickr_gen.next()
						except StopIteration:
							# Give up on fetching images.
							break
		
					# Catch duplicated image sets.
					if imgs == self.flickr_imgs:
						self.flickr_gen = None
						break
		
					# Add the fetched images to our cache.
					self.flickr_imgs += imgs
				else:
					log.warn('Using old Flickr plugin API.')
					# Use the old Flickr plugin API.
					imgs = self.flickr.get_image_list(self.flickrurl)
					urls = [self.flickr.url_list[img] for img in imgs]
					self.flickr_imgs += zip(imgs, urls)
					if len(self.flickr_imgs) == 0:
						break

			if len(self.flickr_imgs) == 0:
				return False

			image, url = random.choice(self.flickr_imgs)
			self.flickr_imgs.remove((image, url))

			self.url = url
			self.__image = os.path.join(self.home, ".slide.jpg")
			self.flickr.save_image(image, self.__image)
			forecast = self.__image

			print self.__image
		except:
			return False

		return True

	def fetch_mediaRSS(self):

		print "Trying RSS..."

		print self.mediaRSS_URL

		try:
			source = urlopen(self.mediaRSS_URL)
			sourcetxt = source.read()

			items = []
			while sourcetxt.find("<item>")>0:
				sourcetxt = sourcetxt[sourcetxt.find("<item>")+1:]
				item = sourcetxt[:sourcetxt.find("</item>")]
				items.append(item)

			item = random.choice(items)

			#item = sourcetxt[sourcetxt.find("<item>"):]
			#item = item[:item.find("</item>")]

			imageurl = item[item.rfind("<link>")+6:]
			imageurl = imageurl[:imageurl.find("</link>")]

			realimage = item[item.rfind("<media:content"):]
			realimage = realimage[realimage.find("url=")+4:]
			realimage = realimage[:realimage.find("/>")]
			if realimage.find(" ") >= 0:
				realimage = realimage[:realimage.find(" ")]

			realimage = realimage.replace('"', '').replace("'", "")

			print realimage

			self.url = imageurl

			imageget = urlopen(realimage)
			imagefile = imageget.read()

			fileObj = open( self.home + "/slide.jpg","w") #// open for for write
			fileObj.write(imagefile)

			fileObj.close()	

		except:
			return False

		self.__image =  self.home + "/slide.jpg"	
		forecast = self.__image

		return True

	def fetch_folder(self):

		print "Trying Folder..."

		imgs = []
		if self.recursive:
			for root, dirs, files in os.walk(self.folders): 
				if len(imgs) > self.recursion_limit:
					break
				for file in files:
					if len(imgs) > self.recursion_limit:
						break
					try:
						if os.path.splitext(file)[1].lower() in self.use_types:
							imgs.append(os.path.join(root,file))
					except:
						pass
		else:
			if os.path.exists(self.folders) and os.path.isdir(self.folders): 
				for f in os.listdir(self.folders):				
					try:  #splitext[1] may fail
						if os.path.splitext(f)[1].lower() in self.use_types: 
							# if so, add it to our list.
									 imgs.append(self.folders + os.sep + f)
					except:
						pass
		try:
			self.__image = random.choice(imgs)
		except:
			pass

		print self.__image

		return True

	def fetch_image(self):
	        threading.Thread(target=self.__fetch_image_thread).start()

	def __fetch_image_thread(self):
		if not self.initialized or self.fetching:
			return False

		self.fetching = True

		try:
			self.__lock.acquire()

			if self.engine1 == 'Flickr':
				if not self.fetch_flickr():
					self.failstate = True
					if self.laststate is not self.failstate:
						self.__notifier.notify("Flickr RSS failed, fallback!")
						self.laststate = self.failstate
				else:
					self.laststate = not self.failstate
					self.failstate = False

			if self.engine1 == 'Media RSS':
				if not self.fetch_mediaRSS():
					self.failstate = True
					if self.laststate is not self.failstate:
						self.__notifier.notify("Media RSS failed, fallback!")
						self.laststate = self.failstate
				else:
					self.laststate = not self.failstate
					self.failstate = False

			if self.engine1 == 'Folder' or self.failstate:
				self.fetch_folder()

		finally:
			self.__lock.release()
		
		self.fetching = False
	
	# --------------------------------------------------------------------------
	# Screenlet handlers
	# --------------------------------------------------------------------------
	def update(self):
		#log.error('Failed to load image "%s": %s (only PNG images supported yet)' % (filename, ex))
		if self.slide:
			self.fetch_image()
		return True

	def on_drag_enter (self, drag_context, x, y, timestamp):
		self.redraw_canvas()

	def on_drag_leave (self, drag_context, timestamp):
		self.redraw_canvas()

	def on_drop (self, x, y, sel_data, timestamp):
		log.debug("Data dropped ...")
		filename = ''
		filename = utils.get_filename_on_drop(sel_data)[0]
		log.debug("Data filename: filename")
		if filename != '':
			#self.set_image(filename)
			self.__image = filename.replace(chr(34),'')

	def show_install_dialog (self):
		"""Craete/Show the install-dialog."""
		# create filter
		flt = gtk.FileFilter()
		flt.add_pattern('*.')

		# create dialog
		dlg = gtk.FileChooserDialog(action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,buttons=(gtk.STOCK_CANCEL, 
			gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dlg.set_current_folder(os.environ['home'])
		dlg.set_title(('Select a folder'))
		dlg.set_filter(flt)
		# run
		resp		= dlg.run()
		filename	= dlg.get_filename()
		dlg.destroy()
		if resp == gtk.RESPONSE_OK:
			# create new installer
			
			# try installing and show result dialog
			self.window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
			self.folders = filename 

			self.window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))	

	def draw_boundingbox(self, ctx):
		bounding_color = (1.0, 0.0, 0.0, 0.5)
		ctx.set_source_rgba(*bounding_color)

		self.draw_rectangle(
		  ctx,0,0,
		  self.bounding_x*self.scale, 
		  self.bounding_y*self.scale)

	def on_draw (self, ctx):

		ctx.set_operator(cairo.OPERATOR_OVER)

		if self.show_bounding:
		  self.draw_boundingbox(ctx)

		image_w, image_h = [200, 200]

		if self.__image is not None and len(self.__image)>0 and self.initialized: 
			image_w, image_h = self.get_image_size(self.__image)

		scaled_image_border = self.scale * self.image_border
		scaled_image_shadow = self.scale * self.image_shadow
		scaled_total_rim = (scaled_image_border + scaled_image_shadow *2 ) * 2.0

		scaled_bounding_x =  self.bounding_x * self.scale
		scaled_bounding_y =  self.bounding_y * self.scale

		image_aspect = (image_w + scaled_total_rim ) / (image_h + scaled_total_rim)
		image_scale = 1.0
		b_box_aspect = (1.0 * self.bounding_x) / (1.0 * self.bounding_y)

		if image_aspect > b_box_aspect:
		  image_scale =  ( scaled_bounding_x - scaled_total_rim /2.0 ) / (1.0 * image_w )
		else:
		  image_scale = ( scaled_bounding_y - scaled_total_rim /2.0) / (1.0 * image_h)

		#background_color = (self.border_r, self.border_g, self.border_b, self.border_a)
		#ctx.set_source_rgba(*background_color) 

		scaled_image_x = image_scale * image_w
		scaled_image_y = image_scale * image_h

		tl_image_corner_x = scaled_total_rim / 4.0 
		tl_image_corner_y = scaled_total_rim / 4.0

		if self.anchor == 'upper right':
		  tl_image_corner_x = scaled_bounding_x - (scaled_total_rim / 4.0  + scaled_image_x)

		if self.anchor == 'lower left':
		  tl_image_corner_y = scaled_bounding_y - (scaled_total_rim / 4.0 + scaled_image_y)

		if self.anchor == 'lower right':
		  tl_image_corner_x = scaled_bounding_x - ( scaled_total_rim / 4.0 + scaled_image_x)
		  tl_image_corner_y = scaled_bounding_y - ( scaled_total_rim / 4.0 + scaled_image_y)

		if self.anchor == 'center':
		    if image_aspect < b_box_aspect:
		        tl_image_corner_x = (scaled_bounding_x - (scaled_total_rim / 4.0 + scaled_image_x - scaled_image_shadow)) / 2.0
		        tl_image_corner_y = (scaled_bounding_y - (scaled_total_rim / 4.0 + scaled_image_y))
		    else:
		        tl_image_corner_x = (scaled_bounding_x - (scaled_total_rim / 4.0 + scaled_image_x)) 
		        tl_image_corner_y = (scaled_bounding_y - (scaled_total_rim / 4.0 + scaled_image_y - scaled_image_shadow)) / 2.0

		#ctx.set_source_rgba(*self.border_color)

		self.draw_rectangle_advanced(
			ctx, 
			tl_image_corner_x - scaled_image_shadow , 
			tl_image_corner_y - scaled_image_shadow , 
			scaled_image_x, 
			scaled_image_y, 
			rounded_angles=(1, 1, 1, 1), 
			fill=False, 
			border_size = scaled_image_border , 
			border_color = self.border_color, 
			shadow_size = scaled_image_shadow , 
			shadow_color = self.shadow_color)

		#self.draw_rounded_rectangle(
		 # ctx,
		 # tl_image_corner_x,
		 # tl_image_corner_y,
		 # 1.0 * scaled_image_border / (self.scale), 
		 # scaled_image_x + 2.0 * scaled_image_border , 
		 # scaled_image_y + 2.0 * scaled_image_border)


		if self.__image is not None and len(self.__image)>0 and self.initialized: 
			self.draw_scaled_image( ctx, 
			  tl_image_corner_x,# + scaled_total_rim / 2.0, 
			  tl_image_corner_y,# + scaled_total_rim / 2.0, 
			  self.__image, 
			  scaled_image_x, 
			  scaled_image_y)

		self.menu_x = tl_image_corner_x + scaled_image_x - 80 - scaled_image_border * 2.0
		self.menu_y = tl_image_corner_y + scaled_image_y - 23 - scaled_image_border * 2.0
		ctx.translate(self.menu_x,self.menu_y)

		if self.paint_menu == True and  self.showbuttons == True: self.theme.render(ctx, 'menu')

		if self.initialized and not self.updated:
			self.updated = True
			self.update()


	def on_focus(self, event):
		self.paint_menu = True
		self.redraw_canvas()

	def on_unfocus(self, event):
		self.paint_menu = False
		self.redraw_canvas()

	def on_mouse_down(self,event):
			x, y = self.window.get_pointer()
			if y >= self.menu_y and y <=self.menu_y+22:
				if x >= self.menu_x and x <= self.menu_x+26 :
					self.slide = False
					self.update()
				elif x >= self.menu_x+27 and x <= self.menu_x+49 :
					self.slide = True
					self.update()
				elif x >= self.menu_x+50 and x <= self.menu_x+76 :
					self.fetch_image()
					self.redraw_canvas()

	def on_menuitem_select (self, id):
		"""handle MenuItem-events in right-click menu"""
		if id == "next":
			# TODO: use DBus-call for this
			#self.switch_hide_show()
			
			self.fetch_image()
			self.redraw_canvas()

		if id == "visit":
			# TODO: use DBus-call for this
			#self.switch_hide_show()
			if self.engine1 == 'Flickr':
				os.system('firefox ' + self.url + " &")
			elif self.engine1 == 'Media RSS':
				os.system('firefox ' + self.url + " &")
			elif self.engine1 == 'Folder':
				os.system('gnome-open ' + chr(34) + self.__image + chr(34) + " &")

		if id == "wall":
			# TODO: use DBus-call for this
			#self.switch_hide_show()
			if self.engine1 == 'Folder':
				os.system("gconftool-2 -t string -s /desktop/gnome/background/picture_filename " + chr(34) + self.__image + chr(34))
				os.system("gconftool-2 -t bool -s /desktop/gnome/background/draw_background False")
				os.system("gconftool-2 -t bool -s /desktop/gnome/background/draw_background True")
			elif self.engine1 == 'Flickr':
				screenlets.show_message(self,'Can only set wallpaper to local images')
			elif self.engine1 == 'Media RSS':
				screenlets.show_message(self,'Can only set wallpaper to local images')

		if id == "start":
			self.slide = True
			self.update()
		
		if id == "stop":
			self.slide = False

		if id[:7] == "Install":
			# TODO: use DBus-call for this
			self.show_install_dialog()
			self.update()

	def on_draw_shape (self, ctx):
		ctx.scale(self.scale, self.scale)
		if self.theme:
			#self.theme['control-bg.svg'].render_cairo(ctx)
			ctx.set_source_rgba(1, 1, 1, 1)
			ctx.rectangle (0, 0, self.width, self.height)
			ctx.fill()

																					
# If the program is run directly or passed as an argument to the python
# interpreter then launch as new application
if __name__ == "__main__":
	# create session object here, the rest is done automagically
	import screenlets.session
	screenlets.session.create_session(SlideshowScreenlet, threading=True)

